<?php
/** 
 * An order item is a product which has been added to an order, 
 * ready for purchase. An order item is typically a product itself,
 * but also can include references to other information such as 
 * product attributes like colour, size, or type.
 * 
 * @package ecommerce
 */
class OrderItem extends OrderAttribute {
	
	protected $_id;
	
	protected $_quantity;
	
	protected $_color;
	
	protected $_size;
	
	static $disable_quantity_js = false;
	
	public static $db = array(
		'Quantity' => 'Int',
		'Color'=>'Varchar(100)',
		'Taglia'=>'Varchar(1)',
	);
	
	public static $casting = array(
		'UnitPrice' => 'Currency',
		'Total' => 'Currency'
	);
	
	public function __construct($object = null, $quantity = 1, $color = NULL, $taglia = NULL) {

		// Case 1: Constructed by getting OrderItem from DB
		if(is_array($object)) {
			$this->_quantity = $object['Quantity'];
			$this->_color = $object['Color'];
			$this->_taglia = $object['Taglia'];
		}
		
		// Case 2: Constructed in memory
		if(is_object($object)) {
			$this->_quantity = $quantity;
			$this->_color = $color;
			$this->_taglia = $taglia;
		}
		
		parent::__construct();
	}
	
	static function disable_quantity_js(){
		self::$disable_quantity_js = true;
	}

	function updateForAjax(array &$js) {
		$total = DBField::create('Currency', $this->Total())->Nice();
		$js[] = array('id' => $this->TableTotalID(), 'parameter' => 'innerHTML', 'value' => $total);
		$js[] = array('id' => $this->CartTotalID(), 'parameter' => 'innerHTML', 'value' => $total);
		$js[] = array('id' => $this->CartQuantityID(), 'parameter' => 'innerHTML', 'value' => $this->getQuantity());
		$js[] = array('name' => $this->QuantityFieldName(), 'parameter' => 'value', 'value' => $this->getQuantity());
		$js[] = array('name' => $this->ColorFieldName(), 'parameter' => 'value', 'value' => $this->getColor());
		$js[] = array('name' => $this->TagliaFieldName(), 'parameter' => 'value', 'value' => $this->getTaglia());
	}
	
	/**
	 * Populate some OrderItem object attributes before
	 * writing them to the OrderItem DB record.
	 * 
	 * PRECONDITION: The order item is not saved in the database yet.
	 */
	function onBeforeWrite() {
		parent::onBeforeWrite();
		
		$this->Quantity = $this->_quantity;
		$this->Color = $this->_color;
		$this->Taglia = $this->_taglia;
	}

	/**
	 * Get the quantity attribute from memory.
	 * @return int
	 */
	public function getQuantity() {
		return $this->_quantity;
	}

	public function getColor(){
		return $this->_color;
	}
	
	public function getTaglia(){
		return $this->_taglia;
	}
	/**
	 * Set the quantity attribute in memory.
	 * PRECONDITION: The order item is not saved in the database yet.
	 * 
	 * @param int $quantity The quantity to set
	 */
	public function setQuantityAttribute($quantity) {
		$this->_quantity = $quantity;
	}

	public function setColorAttribute($color) {
		$this->_color = $color;
	}

	public function getColorName(){
		return DataObject::get('Color',"Color.ID = $this->Color","","","");
	}
	
	public function setTagliaAttribute($taglia) {
		$this->_taglia = $taglia;
	}

	public function getTagliaName(){
		return DataObject::get('Taglia',"Taglia.ID = $this->Taglia","","","");
	}
	/**
	 * Increment the quantity attribute in memory by a given amount.
	 * PRECONDITION: The order item is not saved in the database yet.
	 * 
	 * @param int $quantity The amount to increment the quantity by.
	 */
	public function addQuantityAttribute($quantity) {
		$this->_quantity += $quantity;
	}
	
	function hasSameContent($orderItem) {
		return $orderItem instanceof OrderItem;
	}

	public function debug() {
		$id = $this->ID ? $this->ID : $this->_id;
		$quantity = $this->_quantity;
		$color = $this->_color;
		$taglia =  $this->_taglia;
		$orderID = $this->ID ? $this->OrderID : 'The order has not been saved yet, so there is no ID';
		
		return <<<HTML
			<h2>$this->class</h2> 
			<h3>OrderItem class details</h3>
			<p>
				<b>ID : </b>$id<br/>
				<b>Quantit : </b>$quantity<br/>
				<b>Taglia : </b>$taglia<br/>
				<b>ID Ordine: </b>$orderID
			</p>
HTML;
	}
	
	
	######################
	## TEMPLATE METHODS ##
	######################

	function UnitPrice() {
		user_error("OrderItem::UnitPrice() called. Please implement UnitPrice() on $this->class", E_USER_ERROR);
	}
	
	protected function QuantityFieldName() {
		return $this->MainID() . '_Quantity';
	}

	protected function ColorFieldName() {
		return $this->MainID() . '_Color';
	}
	
	protected function TagliaFieldName() {
		return $this->MainID() . '_Taglia';
	}
	/*
	 * @Depricated - use QuantityField
	 */
	function AjaxQuantityField(){
		return $this->QuantityField();
	}
	
	function QuantityField() {
		if(!self::$disable_quantity_js){
			Requirements::javascript(THIRDPARTY_DIR . '/jquery/jquery.js');
			Requirements::javascript('ecommerce/javascript/ecommerce.js');
		}
		
		$quantityName = $this->QuantityFieldName();
		$setQuantityLinkName = $quantityName . '_SetQuantityLink';
		$setQuantityLink = $this->setquantityLink();
		
		$quantity = ($this->_quantity) ? $this->_quantity : ""; //TODO: make this customisable (ie "0", or "")
		
		return <<<HTML
			<input name="$quantityName" class="ajaxQuantityField" type="text" value="$quantity" size="3" maxlength="3" disabled="disabled"/>
			<input name="$setQuantityLinkName" type="hidden" value="$setQuantityLink"/>
HTML;
	}
	
	function Total() {
		return $this->UnitPrice() * $this->_quantity;
	}

	function TableTitle() {
		return 'Prodotto';
	}
	
	function ProductTitle() {
    if(!isset($_REQUEST['custom'])) {
		  return $this->Product()->Title;
		}
	}
	
	function CartQuantityID() {
		return $this->CartID() . '_Quantity';
	}
	
	function checkoutLink() {
		return CheckoutPage::find_link();
	}

  function OrderFromDb(){
    $custom = $_REQUEST['custom'];
    $params = explode('-', $custom);
    $payment = DataObject::get_by_id('PayPalPayment', $params[0]);
    $order = DataObject::get_by_id('Order', $payment->OrderID);
  }
  
}
?>